#include "boom.h"

namespace boom2x
{

/** Версия библиотеки. */
const char BOOM2X_VERSION[]="0.1.0.1";

const int WINDOW_WIDTH = 320;
const int WINDOW_HEIGHT = 240;
const char WINDOW_TITLE[] = "boom2x window";

//------------------------------------------------------------------------------
// Базовый модуль.
Module::Module()
{
   executing=false;
}

Module::~Module()
{
}

void Module::stopExecution()
{
   executing=false;
}

// Основная исполняющая функция.
void Module::execute()
{
   draw();

   kit.clearEvents();
   executing=true;
   while(executing)
   {
      Event event;
      kit.getEvent(&event);
      switch(event.type)
      {
      case  seNone:
            onIdle();
            break;

      case  seButton:
            onButtonDown(event);
            break;

      case  seStylusDown:
            onStylusDown(event);
            break;

      case  seStylusUp:
            onStylusUp(event);
            break;

      default:
            break;
      }
   }
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
Toolkit kit;

// Основной каталог ресурсов.
const char Toolkit::resources[]="res/";


Toolkit::Toolkit()
{
   screen=NULL;
   joystick=NULL;
   font=NULL;
}

Toolkit::~Toolkit()
{
}

bool Toolkit::start()
{
   // Инициализация SDL.
   SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER);
   // Настроить экран gp2x.
	screen = SDL_SetVideoMode(WINDOW_WIDTH, WINDOW_HEIGHT, 16, SDL_HWSURFACE | SDL_DOUBLEBUF);
	if(!screen)
	{
		// Отладочный вывод в последовательный порт.
		printf("boom2x: Toolkit::start(): couldn't initialize screen.\n");
		return false;
	}

#ifdef GP2X                                        // Если на gp2x, то отключить курсор мыши.
   SDL_ShowCursor(SDL_DISABLE);
#endif

#ifndef GP2X                                       // Если не на gp2x, то установить заголовок окна.
	SDL_WM_SetCaption(WINDOW_TITLE, 0);
#endif

   joystick=SDL_JoystickOpen(0);                   // Инициализировать джойстик. На gp2x - всегда первый (0).

   SDL_EventState(SDL_MOUSEMOTION, SDL_IGNORE);    // "Болтанку" стилуса пока игнорируем.

	TTF_Init();                                     // Включить SDL_TTF.
   setForeColour(0, 0, 0);                         // Установить по умолчанию чёрный цвет.

	printf("boom2x: Toolkit::start()\n");
   return true;
}

void Toolkit::shutdown()
{
   SDL_JoystickClose(joystick);        // Отключить джойстик.
   SDL_FreeSurface(screen);				// Освободить память, занятую под буфер экрана.
	TTF_Quit();									// Отключить SDL_TTF.

#ifndef GP2X
	SDL_Quit();                         // На ББ просто закрываем SDL.
#endif

#ifdef GP2X										// На gp2x надо вернуться в меню.
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#endif
}


// Выбрать следующее сообщение из очереди.
bool Toolkit::getEvent(Event* event)
{
   if(!event)
      return false;

	event->type=seNone;

	SDL_Event e;
	if( SDL_PollEvent(&e) )
	{
	   switch(e.type)
	   {
      case  SDL_KEYDOWN:
            event->type=seButton;
            switch(e.key.keysym.sym)
            {
            case  SDLK_UP     : event->button=jbUp; break;
            case  SDLK_DOWN   : event->button=jbDown; break;
            case  SDLK_LEFT   : event->button=jbLeft; break;
            case  SDLK_RIGHT  : event->button=jbRight; break;
            case  SDLK_ESCAPE : event->button=jbHome; break;
            case  SDLK_RETURN : event->button=jbSelect; break;
            case  SDLK_a      : event->button=jbA; break;
            case  SDLK_b      : event->button=jbB; break;
            case  SDLK_x      : event->button=jbX; break;
            case  SDLK_y      : event->button=jbY; break;

            default:
               event->type=seNone;
               break;
            }
            break;

		case  SDL_JOYBUTTONDOWN:
            event->type=seButton;
            event->button=(JoyButton)e.jbutton.button;
            break;

      case  SDL_MOUSEBUTTONDOWN:
				event->type=seStylusDown;
				event->x=e.button.x;
				event->y=e.button.y;
				break;

      case  SDL_MOUSEBUTTONUP:
				event->type=seStylusUp;
				event->x=e.button.x;
				event->y=e.button.y;
				break;
      } // switch e.type
	} // if SDL_PollEvent

	return true;
}

// Очистить очередь сообщений.
void Toolkit::clearEvents()
{
	SDL_Event e;
	while( SDL_PollEvent(&e) );
}

// Загрузить шрифт заданной гарнитуры кегля size.
TTF_Font* Toolkit::loadFont(const char* filename, uint8 size)
{
	TTF_Font* newfont=TTF_OpenFont(filename, size);
#if PARANOIC
	if(!newfont)
	{
      printf("boom2x: Toolkit::loadFont(): couldn't load font.\n");
      return false;
	}
#endif
   return newfont;
}

// Выгрузить шрифт.
void Toolkit::freeFont(TTF_Font* aFont)
{
   if(font)
      TTF_CloseFont(font);
}

// Установить шрифт.
void Toolkit::setFont(TTF_Font* aFont)
{
   font=aFont;
}


// Установить активный цвет.
void Toolkit::setForeColour(int aR, int aG, int aB)
{
   foreColour.r=aR;
   foreColour.g=aG;
   foreColour.b=aB;
}

// Вывести на поверхность строку заданным цветом.
void Toolkit::drawString(SDL_Surface* surface, const char* value, int x, int y, SDL_Colour colour)
{
#if PARANOIC
	if(!font)
	{
      printf("boom2x: Toolkit::drawString(): no font.\n");
      return;
	}
#endif

	SDL_Surface* textSurface = TTF_RenderText_Blended(font, value, colour);
#if PARANOIC
	if(!textSurface)
	{
      printf("boom2x: Toolkit::drawString(): couldn't create textSurface.\n");
      return;
	}
#endif

   SDL_Rect textLocation = { x, y, 0, 0 };
   SDL_BlitSurface(textSurface, NULL, surface, &textLocation);
   SDL_FreeSurface(textSurface);
}

// Вывести на поверхность строку заданным цветом.
void Toolkit::drawString(SDL_Surface* surface, const char* value, int x, int y)
{
   drawString(surface, value, x, y, foreColour);
}

// Вывести на экран целое заданным цветом.
void Toolkit::drawInt(SDL_Surface* surface, int value, int x, int y, SDL_Colour colour)
{
	char s[16];
	sprintf(s, "%d", value);

   drawString(surface, s, x, y, colour);
}

// Вывести на экран целое.
void Toolkit::drawInt(SDL_Surface* surface, int value, int x, int y)
{
   drawInt(surface, value, x, y, foreColour);
}


void Toolkit::drawSprite(SDL_Surface* surface, SDL_Surface* sprite, int srcX, int srcY, int dstX, int dstY, int width, int height)
{
	SDL_Rect srcRect;
	srcRect.x = srcX;
	srcRect.y = srcY;
	srcRect.w = width;
	srcRect.h = height;

	SDL_Rect dstRect;
	dstRect.x = dstX;
	dstRect.y = dstY;
	dstRect.w = width;
	dstRect.h = height;

   SDL_BlitSurface(sprite, &srcRect, surface, &dstRect);
}
//------------------------------------------------------------------------------

} // namespace boom2x
